/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#ifndef PINGER_FACTORY_H
#define PINGER_FACTORY_H

#include <string>

#include <boost/asio.hpp>
#include <boost/shared_ptr.hpp>

#include "host/pinger.h"
#include "host/pingprotocol.h"
#include "host/pingrotate.h"

//-----------------------------------------------------------------------------
// PingerFactory
//-----------------------------------------------------------------------------

class PingerFactory
{
public:
    static PingerItem createPinger(
            const PingProtocol protocol,
            const IoServiceItem io_serv,
            const std::string &network_interface,
            const int ping_reply_timeout
    );

    static PingRotateItem createPinger(
            const PingProtocolList &protocol_list,
            const IoServiceItem io_serv,
            const std::string &network_interface,
            const std::string &destination_address,
            const uint16_t destination_port,
            const std::string &nameserver,
            const int resolved_ip_ttl_threshold,
            const int ping_reply_timeout
    );

private:
    PingerFactory();
    ~PingerFactory();

};

#endif // PINGER_FACTORY_H
