// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef ICMP_MESSAGE_H
#define ICMP_MESSAGE_H

#include <stdint.h>

#include <istream>
#include <ostream>
#include <boost/utility.hpp>

#include "icmp/icmptype.h"

//-----------------------------------------------------------------------------
// IcmpMessage
//-----------------------------------------------------------------------------

/**
 * @brief Abstract class to which the ICMP messages are interpreted.
 */
class IcmpMessage : boost::noncopyable
{
public:
    void init( const Icmpv4Type type );
    void init( const Icmpv6Type type );

    virtual Icmpv4Type get_type_v4() const = 0;
    virtual void set_type_v4( const Icmpv4Type type ) = 0;

    virtual Icmpv6Type get_type_v6() const = 0;
    virtual void set_type_v6( const Icmpv6Type type ) = 0;

    virtual uint8_t get_code() const = 0;
    virtual void set_code( const uint8_t code ) = 0;

    virtual uint16_t get_checksum() const = 0;
    virtual void set_checksum( const uint16_t checksum ) = 0;

    virtual uint16_t get_identifier() const = 0;
    virtual void set_identifier( const uint16_t identifier ) = 0;

    virtual uint16_t get_sequence_number() const = 0;
    virtual void set_sequence_number( const uint16_t sequence_number ) = 0;

    virtual std::istream& read( std::istream &is ) = 0;
    virtual std::ostream& write( std::ostream &os ) const = 0;

protected:
    IcmpMessage();
    virtual ~IcmpMessage();

};

#endif // ICMP_MESSAGE_H
