// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef ICMP_PACKET_H
#define ICMP_PACKET_H

#include <stdint.h>

#include <iostream>

#include <boost/asio.hpp>
#include <boost/date_time/posix_time/posix_time_types.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

//-----------------------------------------------------------------------------
// IcmpPacket
//-----------------------------------------------------------------------------

/**
 * @brief Abstract class for ICMP Packets.
 */
class IcmpPacket : boost::noncopyable
{
public:
    virtual bool match_echo_reply(
            const uint16_t identifier,
            const uint16_t sequence_number,
            const boost::asio::ip::address &source_address
    ) const = 0;
    virtual bool match_destination_unreachable(
            const uint16_t identifier,
            const uint16_t sequence_number,
            const boost::asio::ip::address &source_address
    ) const = 0;

    virtual void print_echo_reply(
            const std::size_t &bytes_transferred,
            const boost::posix_time::ptime &time_packet_sent
    ) const = 0;
    virtual void print_destination_unreachable() const = 0;

    enum ReadReturnCode {
        ReadReturnCode_OK = 0,
        ReadReturnCode_UNSPECIFIED,
        ReadReturnCode_FAIL,
        ReadReturnCode_NOT_ENOUGH_DATA,
        ReadReturnCode_BAD_STREAM,
        ReadReturnCode_INVALID_SIZE,
        ReadReturnCode_UNKNOWN_PROTOCOL
    };

    static std::string return_code_to_string( const ReadReturnCode &code );

    virtual ReadReturnCode read( std::istream &is ) = 0;
    virtual bool write( std::ostream &os ) const = 0;

    friend std::istream& operator>>(
            std::istream &is,
            IcmpPacket &packet
    );
    friend std::ostream& operator<<(
            std::ostream &os,
            const IcmpPacket &packet
    );

protected:
    IcmpPacket();
    virtual ~IcmpPacket();

};

//-----------------------------------------------------------------------------
// IcmpPacketItem
//-----------------------------------------------------------------------------

typedef boost::shared_ptr<IcmpPacket> IcmpPacketItem;

#endif // ICMP_PACKET_H
